//
//  UIButton+CustomColoring.m
//  LittleNote
//
//  Created by Clark Barry on 1/18/13.
//  Copyright (c) 2013 CSHaus. All rights reserved.
//

#import "UIButton+CustomColoring.h"
#import <QuartzCore/QuartzCore.h>
@interface UIButton (InternalCustomColoring)
+(UIImage *)imageForColor:(UIColor *)color;
@end

@implementation UIButton (CustomColoring)

+(UIImage *)imageForColor:(UIColor *)color{
    CGRect rect = CGRectMake(0, 0, 10, 10);
    UIGraphicsBeginImageContext(rect.size);
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetFillColorWithColor(context, color.CGColor);
    CGContextFillRect(context, rect);
    
    NSArray *colors = @[(id)[[UIColor colorWithWhite:1 alpha:.1]CGColor],(id)[[UIColor colorWithWhite:.8 alpha:.2]CGColor],(id)[[UIColor colorWithWhite:.75 alpha:.2]CGColor],(id)[[UIColor colorWithWhite:.5 alpha:.1]CGColor]];
    CGFloat locations[] = {0,.5,.5,1};
    
    CGGradientRef gradient = CGGradientCreateWithColors(CGColorGetColorSpace((__bridge CGColorRef)([colors objectAtIndex:0])), (__bridge CFArrayRef)colors, locations);
    
    CGPoint top = CGPointMake(CGRectGetMidX(rect), 0);
    CGPoint bottom = CGPointMake(CGRectGetMidX(rect), CGRectGetHeight(rect));
    
    CGContextDrawLinearGradient(context, gradient, top, bottom, 0);
    
    CGGradientRelease(gradient);
    
    UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    
    return image;
}

+(UIButton *)buttonWithBackgroundColor:(UIColor *)backgroundColor andHighlightColor:(UIColor *)highlightColor{
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    [button.layer setMasksToBounds:YES];
    button.layer.borderWidth = .1;
    button.layer.borderColor = [[UIColor darkGrayColor]CGColor];
    button.layer.cornerRadius = 7;
    
    [button setBackgroundImage:[[self class]imageForColor:backgroundColor] forState:UIControlStateNormal];
    [button setBackgroundImage:[[self class]imageForColor:highlightColor] forState:UIControlStateHighlighted];
    
    return button;
}

-(void)setButtonBackgroundColor:(UIColor *)backgroundColor andHighlightColor:(UIColor *)highlightColor{
    [self setBackgroundImage:[[self class]imageForColor:backgroundColor] forState:UIControlStateNormal];
    [self setBackgroundImage:[[self class]imageForColor:highlightColor] forState:UIControlStateHighlighted];
}
@end
