//
//  ThemeManager.m
//  LittleNote
//
//  Created by Clark Barry on 3/12/13.
//  Copyright (c) 2013 CSHaus. All rights reserved.
//

#import "ThemeManager.h"
#import "LittleNoteTheme.h"

static NSString *kThemeIndexKey = @"themeindex";

#define DEFAULT_TEXT_COLOR [UIColor colorWithRed:.99 green:.99 blue:.99 alpha:1]
#define DEFAULT_BACKGROUND_COLOR [UIColor colorWithRed:210./255. green:171./255. blue:254./255. alpha:1]
#define DEFAULT_HIGHLIGHT_COLOR [UIColor colorWithRed:174./255. green:7./255. blue:204./255. alpha:1]


#define BAR_TINT_COLOR [UIColor colorWithRed:190./255. green:151./255. blue:234./255. alpha:1]

@interface ThemeManager (Internal)
-(void)setUpOriginal;
-(void)setUpLegalPad;
-(void)setUpChalkBoard;
-(void)setUpTerminal;
-(void)setUpTablet;
@end

@implementation ThemeManager

@synthesize themes = _themes;
@synthesize currentTheme = _currentTheme;
@synthesize currentIndex = _currentIndex;

static ThemeManager *__instance = nil;

+(ThemeManager *)getInstance{
    if(!__instance){
        static dispatch_once_t onceToken;
        dispatch_once(&onceToken, ^{
            __instance = [[ThemeManager alloc] init];
        });
    }
    return __instance;
}

-(id)init{
    self = [super init];
    if(self){
        _themes = [NSMutableArray arrayWithCapacity:5];
        [self setUpOriginal];
        [self setUpLegalPad];
        [self setUpChalkBoard];
        [self setUpTablet];
        [self setUpTerminal];
        [self setCurrentIndex:[[NSUserDefaults standardUserDefaults] integerForKey:kThemeIndexKey]];
    }
    return self;
}

-(void)setCurrentIndex:(NSInteger)currentIndex{
    _currentIndex = currentIndex;
    _currentTheme = [_themes objectAtIndex:currentIndex];
    [[NSUserDefaults standardUserDefaults]setInteger:_currentIndex forKey:kThemeIndexKey];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

-(void)setUpOriginal{
    LittleNoteTheme *original = [[LittleNoteTheme alloc] initWithName:@"Original"
                                                          navBarColor:BAR_TINT_COLOR
                                                      navBarTextColor:DEFAULT_TEXT_COLOR
                                                            textColor:DEFAULT_TEXT_COLOR
                                                       highlightColor:DEFAULT_HIGHLIGHT_COLOR
                                                      backgroundColor:DEFAULT_BACKGROUND_COLOR
                                                              andFont:@"Helvetica"];
    [_themes addObject:original];
}

-(void)setUpLegalPad{
    LittleNoteTheme *legalPad = [[LittleNoteTheme alloc] initWithName:@"LegalPad"
                                                          navBarColor:[UIColor colorWithRed:160./255. green:82./255. blue:45./255. alpha:1]
                                                      navBarTextColor:DEFAULT_TEXT_COLOR
                                                            textColor:[UIColor colorWithRed:105./255. green:129./255. blue:34./255. alpha:1]
                                                       highlightColor:[UIColor colorWithRed:200./255. green:180./255. blue:123./255. alpha:1]
                                                      backgroundColor:[UIColor colorWithRed:245./255. green:238./255. blue:168./255. alpha:1]
                                                              andFont:@"AmericanTypewriter-Bold"];
    [_themes addObject:legalPad];
}

-(void)setUpChalkBoard{
    LittleNoteTheme *chalkBoard = [[LittleNoteTheme alloc] initWithName:@"ChalkBoard"
                                                            navBarColor:[UIColor colorWithWhite:0 alpha:1]
                                                        navBarTextColor:[UIColor colorWithWhite:1 alpha:1]
                                                              textColor:[UIColor colorWithWhite:.95 alpha:1]
                                                         highlightColor:[UIColor colorWithWhite:.05 alpha:1]
                                                        backgroundColor:[UIColor colorWithWhite:.2 alpha:1]
                                                                andFont:@"ChalkboardSE-Bold"];
    [_themes addObject:chalkBoard];
}

-(void)setUpTerminal{
    LittleNoteTheme *terminal = [[LittleNoteTheme alloc] initWithName:@"Terminal"
                                                          navBarColor:[UIColor colorWithWhite:.15 alpha:1]
                                                      navBarTextColor:[UIColor colorWithRed:0 green:240./255. blue:0 alpha:1]
                                                            textColor:[UIColor colorWithRed:0 green:240./255. blue:0 alpha:1]
                                                       highlightColor:[UIColor colorWithWhite:.05 alpha:1]
                                                      backgroundColor:[UIColor colorWithWhite:.25 alpha:1]
                                                              andFont:@"Symbol"];
    [_themes addObject:terminal];
}

-(void)setUpTablet{
    LittleNoteTheme *tablet = [[LittleNoteTheme alloc] initWithName:@"StoneTablet"
                                                        navBarColor:[UIColor colorWithRed:110./255. green:123./255. blue:139./255. alpha:1]
                                                    navBarTextColor:[UIColor colorWithRed:.4 green:.4 blue:.45 alpha:1]
                                                          textColor:[UIColor colorWithRed:.8 green:.8 blue:.8 alpha:1]
                                                     highlightColor:[UIColor colorWithRed:.3 green:.3 blue:.3 alpha:1]
                                                    backgroundColor:[UIColor colorWithRed:90./255. green:105./255. blue:115./255. alpha:1]
                                                            andFont:@"MarkerFelt-Thin"];
    [_themes addObject:tablet];
}

@end
