//
//  ReadViewController.m
//  ForClark
//
//  Created by Clark Barry on 12/21/12.
//  Copyright (c) 2012 CSHaus. All rights reserved.
//

#import "ReadViewController.h"
#import "NoteManager.h"
#import "Note.h"
#import "NoteView.h"
#import "UIView+UIViewConvenience.h"
#import "User.h"
#import "ThemeManager.h"
#import "LittleNoteTheme.h"

@interface ReadViewController (Internal)
-(void)notesDidUpdate;
-(void)didNotFindNotes;
-(void)configureViewsAtIndex:(NSUInteger)index animated:(BOOL)animated;
-(void)saveCurrentIndex;
@end

static NSString *kCurrentIndexKey = @"Current Index key";

@implementation ReadViewController

-(id)init{
    self = [super init];
    if(self){
        _currentIndex = [[NSUserDefaults standardUserDefaults] integerForKey:kCurrentIndexKey];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notesDidUpdate) name:kNewNotesNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didNotFindNotes) name:kNoNewNotesNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(saveCurrentIndex) name:UIApplicationWillResignActiveNotification object:nil];
        self.title = @"Read";
        [self.tabBarItem setImage:[UIImage imageNamed:@"LittleNoteReadIcon.png"]];
    }
    return self;
}

-(void)saveCurrentIndex{
    [[NSUserDefaults standardUserDefaults] setInteger:_currentIndex forKey:kCurrentIndexKey];
    [[NSUserDefaults standardUserDefaults]synchronize];
}

-(void)loadView{
    [super loadView];

    _noteViewer = [[UIScrollView alloc]initWithFrame:self.view.bounds];
    [_noteViewer setDelegate:self];
    [_noteViewer setContentSize:CGSizeMake(self.view.bounds.size.width*3, self.view.bounds.size.height)];
    [_noteViewer setContentOffset:CGPointZero];
    [_noteViewer setPagingEnabled:YES];
    
    CGRect noteViewRect = CGRectMake(0, 0, self.view.frame.size.width, self.view.frame.size.height);
    _leftNoteView = [[NoteView alloc]initWithFrame:noteViewRect];
    [_noteViewer addSubview:_leftNoteView];
    
    noteViewRect.origin.x += noteViewRect.size.width;
    _middleNoteView = [[NoteView alloc]initWithFrame:noteViewRect];
    [_noteViewer addSubview:_middleNoteView];
    
    noteViewRect.origin.x += noteViewRect.size.width;
    _rightNoteView = [[NoteView alloc]initWithFrame:noteViewRect];
    [_noteViewer addSubview:_rightNoteView];
    
     [self.view addSubview:_noteViewer];
}

-(void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    [self.view setBackgroundColor:theme.backgroundColor];
    
    _notes = [NSArray arrayWithArray:[[NoteManager getInstance] notes]];
    
    CGRect noteViewRect = CGRectMake(0, 0, self.view.frame.size.width, self.view.frame.size.height);
    _leftNoteView.frame = noteViewRect;

    noteViewRect.origin.x += noteViewRect.size.width;

    _middleNoteView.frame = noteViewRect;
    noteViewRect.origin.x += noteViewRect.size.width;
    _rightNoteView.frame = noteViewRect;
    [self configureViewsAtIndex:_currentIndex animated:NO];
    NSInteger offset = _currentIndex % 3;
    _noteViewer.contentOffset = CGPointMake(self.view.frame.size.width * offset, 0);
}

-(void)configureViewsAtIndex:(NSUInteger)index animated:(BOOL)animated{
    NSUInteger notesCount = [_notes count];
    CGSize size = self.view.frame.size;
    if(index == 0 || notesCount == 0 || notesCount == 1 || ![[User getInstance]isSharing]){
        if(![[User getInstance] isSharing]){
            [_leftNoteView configureForNotSharing:size];
            [_middleNoteView configureForRefresh:size];
            [_rightNoteView configureForBlank:size];
        }else if(notesCount == 0){
            [_leftNoteView configureForEmptyWithSize:size];
            [_middleNoteView configureForRefresh:size];
            [_rightNoteView configureForBlank:size];
        }else{
            animated ? [_leftNoteView fadeInNote:[_notes objectAtIndex:0] withSize:size]:[_leftNoteView configureForNote:[_notes objectAtIndex:0] withSize:size];
            
            if(notesCount == 1){
                [_middleNoteView configureForRefresh:size];
            }else{
                animated ? [_middleNoteView fadeInNote:[_notes objectAtIndex:1] withSize:size]:[_middleNoteView configureForNote:[_notes objectAtIndex:1] withSize:size];
                if(notesCount == 2){
                    [_rightNoteView configureForRefresh:size];
                }else{
                    animated ? [_rightNoteView fadeInNote:[_notes objectAtIndex:2] withSize:size]:[_rightNoteView configureForNote:[_notes objectAtIndex:2] withSize:size];
                }
            }
        }
    }else if(index == (notesCount-1) || index == notesCount){
        animated ? [_leftNoteView fadeInNote:[_notes objectAtIndex:(notesCount - 2)] withSize:size]:[_leftNoteView configureForNote:[_notes objectAtIndex:(notesCount-2)] withSize:size];
        animated ? [_middleNoteView fadeInNote:[_notes objectAtIndex:(notesCount-1)] withSize:size]:[_middleNoteView configureForNote:[_notes objectAtIndex:(notesCount-1)] withSize:size];
        [_rightNoteView configureForRefresh:size];
    }else{
        animated ? [_leftNoteView fadeInNote:[_notes objectAtIndex:(index-1)] withSize:size]:[_leftNoteView configureForNote:[_notes objectAtIndex:(index-1)] withSize:size];
        animated ? [_middleNoteView fadeInNote:[_notes objectAtIndex:(index)] withSize:size]:[_middleNoteView configureForNote:[_notes objectAtIndex:(index)] withSize:size];
        animated ? [_rightNoteView fadeInNote:[_notes objectAtIndex:index+1] withSize:size]:[_rightNoteView configureForNote:[_notes objectAtIndex:(index+1)] withSize:size];
    }
    
    if(![[User getInstance]isSharing]){
        _noteViewer.contentSize = CGSizeMake(_noteViewer.frame.size.width, _noteViewer.frame.size.height);
    }else if([_notes count] < 2){
        _noteViewer.contentSize = CGSizeMake(_noteViewer.frame.size.width*2, _noteViewer.frame.size.height);
    }else{
        _noteViewer.contentSize = CGSizeMake(_noteViewer.frame.size.width*3, _noteViewer.frame.size.height);
    }
}

-(void)scrollViewDidScroll:(UIScrollView *)scrollView{
    if((_noteViewer.contentOffset.x <= 0 && _currentIndex > 0) || (_noteViewer.contentOffset.x <= _noteViewer.frame.size.width && _currentIndex == [_notes count] && [_notes count])){
        _currentIndex -= 1;
        if(_currentIndex > 0){
            [_noteViewer setContentOffset:CGPointMake(_noteViewer.frame.size.width, 0)];
        }
        [self configureViewsAtIndex:_currentIndex animated:NO];
    }else if((_noteViewer.contentOffset.x >= _noteViewer.frame.size.width*2 && _currentIndex < [_notes count]) || (_noteViewer.contentOffset.x >= _noteViewer.frame.size.width && _currentIndex == 0)){
        _currentIndex += 1;
        if(_currentIndex < [_notes count]){
             [_noteViewer setContentOffset:CGPointMake(_noteViewer.frame.size.width, 0)];       
        }
        [self configureViewsAtIndex:_currentIndex animated:NO];
    }

}

-(void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView{
    if(_currentIndex >= [_notes count]){
        if(![[NoteManager getInstance] attemptingFetch]){
            NoteView *refreshView = [_notes count] >= 2 ? _rightNoteView : _middleNoteView;
            [refreshView showChecking];
            [[NoteManager getInstance] fetchNotes];
        }
    }
}

-(void)didNotFindNotes{
    dispatch_async(dispatch_get_main_queue(), ^{
        NoteView *refreshView = [_notes count] >= 2 ? _rightNoteView : _middleNoteView;
        [refreshView finishedChecking:NO];
    });
}

-(void)notesDidUpdate{
        dispatch_async(dispatch_get_main_queue(), ^{
            _notes = [[NoteManager getInstance] notes];
            [_noteViewer setContentOffset:CGPointMake(_noteViewer.frame.size.width, 0) animated:YES];
            [self configureViewsAtIndex:_currentIndex animated:YES];
        });
}

-(void)dealloc{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
}

@end
