//
//  SettingsViewController.m
//  LittleNote
//
//  Created by Clark Barry on 1/31/13.
//  Copyright (c) 2013 CSHaus. All rights reserved.
//

#import "SettingsViewController.h"
#import "UIButton+CustomColoring.h"
#import "UITextField+FadeField.h"
#import "UIView+UIViewConvenience.h"
#import "User.h"
#import "Request.h"
#import "ThemeManager.h"
#import "LittleNoteTheme.h"
#import "CommonDefines.h"

static NSString *kCellID = @"cell id";

@interface SettingsViewController ()
-(void)setupRequestResponseButton;
-(void)setupRequestField;
-(void)didFinishEditing;
-(void)removeSelf;
-(void)askToStopFollowing;
-(void)shareResponseButtonPressed;
-(void)logoutButtonPressed;
-(void)newRequest;
-(void)fadeInRequestButton;
-(void)fadeInRequestTextView;
-(void)stoppedFollowing;
-(void)requestProcessed;
-(void)noSuchUser;
-(void)requestCancelled;
-(NSArray *)getConstraints;
@end

@implementation SettingsViewController

-(id)init{
    self = [super init];
    if(self){
        self.title = @"Settings";
        [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(newRequest) name:kNewRequestNotification object:nil];
        [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(requestProcessed) name:kRequestResponseSuccessful object:nil];
        [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(stoppedFollowing) name:kStopSharingSuccussfulNotification object:nil];
        [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(noSuchUser) name:kUserDoesNotExistNotification object:nil];
        [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(requestCancelled) name:kRequestCancelledNotification object:nil];
    }
    return self;
}

-(void)setupRequestResponseButton{
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    if(!_shareResponseButton){
        _shareResponseButton = [UIButton buttonWithBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
        [_shareResponseButton setTitleColor:theme.textColor forState:UIControlStateNormal];
        [_shareResponseButton setTranslatesAutoresizingMaskIntoConstraints:NO];
        [_shareResponseButton addTarget:self action:@selector(shareResponseButtonPressed) forControlEvents:UIControlEventTouchUpInside];
    }else{
        [_shareResponseButton setButtonBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
        [_shareResponseButton setTitleColor:theme.textColor forState:UIControlStateNormal];
    }
    Request *r = [[User getInstance] firstRequest];
    NSString *buttonTitle = [NSString stringWithFormat:@"Share with %@?",r.requesterName];
    [_shareResponseButton setTitle:buttonTitle forState:UIControlStateNormal];
}

-(void)setupRequestField{
    if(!_sharingTextField){
        _sharingTextField = [[UITextField alloc] init];
        [_sharingTextField setDelegate:self];
        [_sharingTextField setPlaceholder:@"Share with someone!"];
        [_sharingTextField setTranslatesAutoresizingMaskIntoConstraints:NO];
        [_sharingTextField setClearButtonMode:UITextFieldViewModeAlways];
        [_sharingTextField setBorderStyle:UITextBorderStyleRoundedRect];
        [_sharingTextField setAutocapitalizationType:UITextAutocapitalizationTypeNone];
        [_sharingTextField setAutocorrectionType:UITextAutocorrectionTypeNo];
        [_sharingTextField setSpellCheckingType:UITextSpellCheckingTypeNo];
    }
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    [_sharingTextField setBackgroundColor:theme.backgroundColor];
    [_sharingTextField setTextColor:theme.textColor];
    [_sharingTextField setFont:[UIFont fontWithName:theme.fontName size:18]];
    if([[User getInstance] isSharing]){
        NSString *sharingWith = [[User getInstance]sharingUsername];
        [_sharingTextField setText:sharingWith];
    }
}

-(void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [[User getInstance]checkRequests];
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    [self.view setBackgroundColor:theme.backgroundColor];
    
    [_logoutButton setButtonBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
    [_logoutButton setTitleColor:theme.textColor forState:UIControlStateNormal];
}

-(void)loadView{
    [super loadView];

    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    
    UIBarButtonItem *rightItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(didFinishEditing)];
    
    [self.navigationItem setRightBarButtonItem:rightItem animated:YES];
    
    _shareLabel = [[UILabel alloc]init];
    [_shareLabel setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_shareLabel setText:@"Sharing with:"];
    [_shareLabel setFont:[UIFont fontWithName:theme.fontName size:18]];
    [_shareLabel setTextColor:theme.textColor];
    [_shareLabel setBackgroundColor:theme.backgroundColor];
    [self.view addSubview:_shareLabel];
    
    if([[User getInstance] hasRequests]){
        [self setupRequestResponseButton];
        _shareView = _shareResponseButton;
    }else{
        [self setupRequestField];
        _shareView = _sharingTextField;
    }
    [self.view addSubview:_shareView];
    
    _themeLabel = [[UILabel alloc] init];
    [_themeLabel setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_themeLabel setText:@"Themes:"];
    [_themeLabel setFont:[UIFont fontWithName:theme.fontName size:18]];
    [_themeLabel setTextColor:theme.textColor];
    [_themeLabel setBackgroundColor:theme.backgroundColor];
    [self.view addSubview:_themeLabel];
    
    
    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    layout.itemSize = CGSizeMake(30, 30);
    layout.sectionInset = UIEdgeInsetsMake(5, 5, 5, 5);
    _themeView = [[UICollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:layout];
    [_themeView registerClass:[UICollectionViewCell class] forCellWithReuseIdentifier:kCellID];
    [_themeView setTranslatesAutoresizingMaskIntoConstraints:NO];
    _themeView.dataSource = self;
    _themeView.delegate = self;
    [_themeView setBackgroundColor:theme.backgroundColor];
    [self.view addSubview:_themeView];
    
    _logoutButton = [UIButton buttonWithBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
    [_logoutButton setTitle:@"Logout" forState:UIControlStateNormal];
    [_logoutButton setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_logoutButton addTarget:self action:@selector(logoutButtonPressed) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:_logoutButton];
    
    [self.view addConstraints:[self getConstraints]];
}

-(NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section{
    return [[[ThemeManager getInstance]themes] count];
}

-(UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    UICollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:kCellID forIndexPath:indexPath];
    CGSize cellSize = cell.frame.size;
    UIImage *themeImage = [[[[ThemeManager getInstance]themes] objectAtIndex:indexPath.row] getImageWithSize:CGSizeMake(cellSize.width, cellSize.height)];
    UIImageView *imageView = [[UIImageView alloc] initWithFrame:cell.bounds];
    if(indexPath.row == [[ThemeManager getInstance] currentIndex]){
        [imageView addBorderWithColor:[UIColor whiteColor] andRadius:2];
    }else{
        [imageView addBorderWithColor:[UIColor blackColor] andRadius:2];
    }
    [imageView setContentMode:UIViewContentModeScaleAspectFill];
    [imageView setClipsToBounds:YES];
    [imageView setImage:themeImage];
    [cell.contentView addSubview:imageView];
    
    return cell;
}

-(void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath{
    [[ThemeManager getInstance] setCurrentIndex:indexPath.row];
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    [self.view setBackgroundColor:theme.backgroundColor];
    
    [_shareLabel setBackgroundColor:theme.backgroundColor];
    [_shareLabel setTextColor:theme.textColor];
    [_shareLabel setFont:[UIFont fontWithName:theme.fontName size:18]];
    
    [_shareView setBackgroundColor:theme.backgroundColor];
    
    [_sharingTextField setTextColor:theme.textColor];
    [_sharingTextField setFont:[UIFont fontWithName:theme.fontName size:18]];

    [_shareResponseButton setButtonBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
    [_shareResponseButton setTitleColor:theme.textColor forState:UIControlStateNormal];
    
    [_themeLabel setBackgroundColor:theme.backgroundColor];
    [_themeLabel setTextColor:theme.textColor];
    [_themeLabel setFont:[UIFont fontWithName:theme.fontName size:18]];
    
    [_themeView setBackgroundColor:theme.backgroundColor];
    
    [_logoutButton setButtonBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
    [_logoutButton setTitleColor:theme.textColor forState:UIControlStateNormal];
    [self.navigationController.navigationBar setTintColor:theme.navBarColor];
    [_themeView reloadData];
}

-(BOOL)textFieldShouldReturn:(UITextField *)textField{
    [textField resignFirstResponder];
    return YES;
}

-(void)textFieldDidEndEditing:(UITextField *)textField{
    [[User getInstance] requestShareWithUser:textField.text];
}

-(BOOL)textFieldShouldClear:(UITextField *)textField{
    [self askToStopFollowing];
    return NO;
}

-(void)askToStopFollowing{
    UIAlertView *a = [[UIAlertView alloc]initWithTitle:@"Stop following?"
                                               message:nil
                                              delegate:self
                                     cancelButtonTitle:@"Cancel"
                                     otherButtonTitles:@"Confirm",nil];
    [a show];

}

-(void)stoppedFollowing{
    if([[User getInstance] hasRequests]){
        [self fadeInRequestButton];
    }else{
        [self fadeInRequestTextView];
    }
}

-(void)fadeInRequestButton{
    dispatch_async(dispatch_get_main_queue(), ^{
        [self setupRequestResponseButton];
        _shareResponseButton.frame = _sharingTextField.frame;
        _shareResponseButton.alpha = 0;
        [self.view addSubview:_shareResponseButton];
        [UIView animateWithDuration:.3
                         animations:^{
                             _sharingTextField.alpha = 0;
                             _shareResponseButton.alpha = 1;
                         } completion:^(BOOL finished) {
                             [_sharingTextField removeFromSuperview];
                             _shareView = _shareResponseButton;
                         }];
    });
}

-(void)requestCancelled{
    dispatch_async(dispatch_get_main_queue(), ^{
        UIAlertView *a = [[UIAlertView alloc]initWithTitle:@"The request has been cancelled"
                                                   message:nil
                                                  delegate:nil
                                         cancelButtonTitle:@"Cancel"
                                         otherButtonTitles:nil];
        [a show];
        if([[User getInstance] hasRequests]){
            [self fadeInRequestButton];
        }else{
            [self fadeInRequestTextView];
        }
    });
}

-(void)noSuchUser{
    dispatch_async(dispatch_get_main_queue(), ^{
        UIAlertView *a = [[UIAlertView alloc]initWithTitle:@"User does not exist!"
                                                   message:nil
                                                  delegate:nil
                                         cancelButtonTitle:@"Cancel"
                                         otherButtonTitles:nil];
        [a show];
    });

}

-(void)fadeInRequestTextView{
    dispatch_async(dispatch_get_main_queue(), ^{
    [self setupRequestField];
    _sharingTextField.frame = _shareResponseButton.frame;
    [self.view addSubview:_sharingTextField];
    _sharingTextField.alpha = 0;
    [UIView animateWithDuration:.3
                     animations:^{
                         _sharingTextField.alpha = 1;
                         _shareResponseButton = 0;
                     } completion:^(BOOL finished) {
                         [_shareResponseButton removeFromSuperview];
                         _shareView = _sharingTextField;
                         [_shareView setNeedsDisplay];
                     }];
    });
}

-(void)requestProcessed{
    if([[User getInstance] hasRequests]){
        [self fadeInRequestButton];
    }else{
        [self fadeInRequestTextView];
    }

}

-(void)newRequest{
    [self fadeInRequestButton];
}

-(void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex{
    if(buttonIndex == 1){
        User *u = [User getInstance];
        [u stopSharing];
        if([u hasRequests]){
            [self fadeInRequestButton];
        }else{
            [_sharingTextField fadeInText:@""];
        }
    }
}

-(void)shareResponseButtonPressed{
    UIActionSheet *a = [[UIActionSheet alloc]initWithTitle:@"Share?"
                                                  delegate:self
                                         cancelButtonTitle:@"Cancel"
                                    destructiveButtonTitle:nil
                                         otherButtonTitles:@"Sounds good",@"I'll pass", nil];
    [a showInView:self.view];
}

-(void)logoutButtonPressed{
    [[User getInstance] logout];
    [self removeSelf];
}

-(void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex{
    switch (buttonIndex) {
        case 0:{
            User *u = [User getInstance];
            Request *r = [u firstRequest];
            [u respondToRequest:r withResponse:YES];
            //update views
            break;
        }case 1:{
            User *u = [User getInstance];
            Request * r = [u firstRequest];
            [u respondToRequest:r withResponse:NO];
            break;
        }default:
            break;
    }
    [actionSheet dismissWithClickedButtonIndex:buttonIndex animated:YES];
}
-(void)didFinishEditing{
    [self removeSelf];
}

-(void)removeSelf{
    if(self.delegate && [self.delegate respondsToSelector:@selector(settingsDidFinish:)]){
        [self.delegate settingsDidFinish:self];
    }
}

-(NSArray *)getConstraints{
    NSMutableArray *constraints = [NSMutableArray arrayWithCapacity:5];
    
    NSLayoutConstraint *shareLabelCenter = [NSLayoutConstraint constraintWithItem:_shareLabel
                                                                        attribute:NSLayoutAttributeCenterX
                                                                        relatedBy:NSLayoutRelationEqual
                                                                           toItem:self.view
                                                                        attribute:NSLayoutAttributeCenterX
                                                                       multiplier:1.0
                                                                         constant:0];
    [constraints addObject:shareLabelCenter];
    
    NSLayoutConstraint *shareLabelWidth = [NSLayoutConstraint constraintWithItem:_shareLabel
                                                                       attribute:NSLayoutAttributeWidth
                                                                       relatedBy:NSLayoutRelationEqual
                                                                          toItem:self.view
                                                                       attribute:NSLayoutAttributeWidth
                                                                      multiplier:(2./3.)
                                                                        constant:0];
    [constraints addObject:shareLabelWidth];
    
    NSLayoutConstraint *shareLabelHeight = [NSLayoutConstraint constraintWithItem:_shareLabel
                                                                        attribute:NSLayoutAttributeHeight
                                                                        relatedBy:NSLayoutRelationEqual
                                                                           toItem:self.view
                                                                        attribute:NSLayoutAttributeHeight
                                                                       multiplier:(1./12.)
                                                                         constant:0];
    [constraints addObject:shareLabelHeight];
    
    NSLayoutConstraint *shareLabelY = [NSLayoutConstraint constraintWithItem:_shareLabel
                                                                   attribute:NSLayoutAttributeCenterY
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:self.view
                                                                   attribute:NSLayoutAttributeCenterY
                                                                  multiplier:(1./6.)
                                                                    constant:0];
    [constraints addObject:shareLabelY];
    
    NSLayoutConstraint *sharingCenter = [NSLayoutConstraint constraintWithItem:_shareView
                                                                      attribute:NSLayoutAttributeCenterX
                                                                      relatedBy:NSLayoutRelationEqual
                                                                         toItem:self.view
                                                                      attribute:NSLayoutAttributeCenterX
                                                                     multiplier:1.0
                                                                       constant:0];
    [constraints addObject:sharingCenter];
    
    NSLayoutConstraint *sharingWidth = [NSLayoutConstraint constraintWithItem:_shareView
                                                                     attribute:NSLayoutAttributeWidth
                                                                     relatedBy:NSLayoutRelationEqual
                                                                        toItem:self.view
                                                                     attribute:NSLayoutAttributeWidth
                                                                    multiplier:(2./3.)
                                                                      constant:0];
    [constraints addObject:sharingWidth];
    
    NSLayoutConstraint *sharingTop = [NSLayoutConstraint constraintWithItem:_shareView
                                                                   attribute:NSLayoutAttributeTop
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:_shareLabel
                                                                   attribute:NSLayoutAttributeBottom
                                                                  multiplier:1.0
                                                                    constant:PADDING];
    [constraints addObject:sharingTop];
    
    NSLayoutConstraint *sharingHeight = [NSLayoutConstraint constraintWithItem:_shareView
                                                                      attribute:NSLayoutAttributeHeight
                                                                      relatedBy:NSLayoutRelationEqual
                                                                         toItem:self.view
                                                                      attribute:NSLayoutAttributeHeight
                                                                     multiplier:(1./12.)
                                                                       constant:0];
    [constraints addObject:sharingHeight];
    
    NSLayoutConstraint *themeLabelX = [NSLayoutConstraint constraintWithItem:_themeLabel
                                                                   attribute:NSLayoutAttributeCenterX
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:self.view
                                                                   attribute:NSLayoutAttributeCenterX
                                                                  multiplier:1.0
                                                                    constant:0];
    [constraints addObject:themeLabelX];
    
    NSLayoutConstraint *themeLabelWidth = [NSLayoutConstraint constraintWithItem:_themeLabel
                                                                       attribute:NSLayoutAttributeWidth
                                                                       relatedBy:NSLayoutRelationEqual
                                                                          toItem:self.view
                                                                       attribute:NSLayoutAttributeWidth
                                                                      multiplier:(2./3.)
                                                                        constant:0];
    [constraints addObject:themeLabelWidth];
    
    NSLayoutConstraint *themeLabelHeight = [NSLayoutConstraint constraintWithItem:_themeLabel
                                                                        attribute:NSLayoutAttributeHeight
                                                                        relatedBy:NSLayoutRelationEqual
                                                                           toItem:self.view
                                                                        attribute:NSLayoutAttributeHeight
                                                                       multiplier:(1./12.)
                                                                         constant:0];
    [constraints addObject:themeLabelHeight];
    
    NSLayoutConstraint *themeLabelTop = [NSLayoutConstraint constraintWithItem:_themeLabel
                                                                     attribute:NSLayoutAttributeTop
                                                                     relatedBy:NSLayoutRelationEqual
                                                                        toItem:_shareView
                                                                     attribute:NSLayoutAttributeBottom
                                                                    multiplier:1.0
                                                                      constant:PADDING];
    [constraints addObject:themeLabelTop];
    
    NSLayoutConstraint *themesCenter = [NSLayoutConstraint constraintWithItem:_themeView
                                                                    attribute:NSLayoutAttributeCenterX
                                                                    relatedBy:NSLayoutRelationEqual
                                                                       toItem:self.view
                                                                    attribute:NSLayoutAttributeCenterX
                                                                   multiplier:1.0
                                                                     constant:0];
    [constraints addObject:themesCenter];
    
    NSLayoutConstraint *themesWidth = [NSLayoutConstraint constraintWithItem:_themeView
                                                                   attribute:NSLayoutAttributeWidth
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:self.view
                                                                   attribute:NSLayoutAttributeWidth
                                                                  multiplier:(2./3.)
                                                                    constant:0];
    [constraints addObject:themesWidth];
    
    NSLayoutConstraint *themesTop = [NSLayoutConstraint constraintWithItem:_themeView
                                                                 attribute:NSLayoutAttributeTop
                                                                 relatedBy:NSLayoutRelationEqual
                                                                    toItem:_themeLabel
                                                                 attribute:NSLayoutAttributeBottom
                                                                multiplier:1.0
                                                                  constant:PADDING];
    [constraints addObject:themesTop];
    
    NSLayoutConstraint *themesHeight = [NSLayoutConstraint constraintWithItem:_themeView
                                                                    attribute:NSLayoutAttributeHeight
                                                                    relatedBy:NSLayoutRelationEqual
                                                                       toItem:_shareView
                                                                    attribute:NSLayoutAttributeHeight
                                                                   multiplier:2.0
                                                                     constant:0];
    [constraints addObject:themesHeight];
    
    
    NSLayoutConstraint *logoutCenter = [NSLayoutConstraint constraintWithItem:_logoutButton
                                                                    attribute:NSLayoutAttributeCenterX
                                                                    relatedBy:NSLayoutRelationEqual
                                                                       toItem:self.view
                                                                    attribute:NSLayoutAttributeCenterX
                                                                   multiplier:1.0
                                                                     constant:0];
    [constraints addObject:logoutCenter];
    
    NSLayoutConstraint *layoutBottom = [NSLayoutConstraint constraintWithItem:_logoutButton
                                                                    attribute:NSLayoutAttributeBottom
                                                                    relatedBy:NSLayoutRelationEqual
                                                                       toItem:self.view
                                                                    attribute:NSLayoutAttributeBottom
                                                                   multiplier:1.0
                                                                     constant:-PADDING];
    [constraints addObject:layoutBottom];
    
    NSLayoutConstraint *logoutWidth = [NSLayoutConstraint constraintWithItem:_logoutButton
                                                                   attribute:NSLayoutAttributeWidth
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:self.view
                                                                   attribute:NSLayoutAttributeWidth
                                                                  multiplier:(2./3.)
                                                                    constant:0];
    [constraints addObject:logoutWidth];
    
    NSLayoutConstraint *logoutHeight = [NSLayoutConstraint constraintWithItem:_logoutButton
                                                                    attribute:NSLayoutAttributeHeight
                                                                    relatedBy:NSLayoutRelationEqual
                                                                       toItem:self.view
                                                                    attribute:NSLayoutAttributeHeight
                                                                   multiplier:(1./12.)
                                                                     constant:0];
    [constraints addObject:logoutHeight];
    
    return constraints;
}


-(void)dealloc{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
}
@end
