//
//  SignInViewController.m
//  LittleNote
//
//  Created by Clark Barry on 12/23/12.
//  Copyright (c) 2012 CSHaus. All rights reserved.
//

#import "SignInViewController.h"
#import "CommonDefines.h"
#import "UIView+UIViewConvenience.h"
#import "UIButton+CustomColoring.h"
#import <QuartzCore/QuartzCore.h>
#import "User.h"
#import "SignUpViewController.h"
#import "ThemeManager.h"
#import "LittleNoteTheme.h"

@interface SignInViewController ()
-(NSArray *)getConstraints;
-(void)submitButtonPressed;
-(void)registerButtonPressed;
-(void)loginSuccessful;
-(void)authenticationFailed;
-(void)connectionFailed;
@end

@implementation SignInViewController

-(id)init{
    self = [super init];
    if(self){
        self.title = @"Sign in";
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(loginSuccessful) name:kLoginSuccessfulNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(authenticationFailed) name:kAuthenticaitonFailedNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectionFailed) name:kConnectionFailedNotification object:nil];
    }
    return self;
}

-(void)loadView{
    [super loadView];
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    
    self.navigationItem.hidesBackButton = YES;
    
    _userNameField = [[UITextField alloc]init];
    [_userNameField setPlaceholder:@"username"];
    [_userNameField setAutocapitalizationType:UITextAutocapitalizationTypeNone];
    [_userNameField setAutocorrectionType:UITextAutocorrectionTypeNo];
    [_userNameField setSpellCheckingType:UITextSpellCheckingTypeNo];
    [_userNameField setBorderStyle:UITextBorderStyleRoundedRect];
    [_userNameField setBackgroundColor:theme.backgroundColor];
    [_userNameField setTextColor:theme.textColor];
    [_userNameField setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_userNameField setDelegate:self];
    [self.view addSubview:_userNameField];
    
    _passwordNameField = [[UITextField alloc]init];
    [_passwordNameField setBorderStyle:UITextBorderStyleRoundedRect];
    [_passwordNameField setBackgroundColor:theme.backgroundColor];
    [_passwordNameField setTextColor:theme.textColor];
    [_passwordNameField setPlaceholder:@"password"];
    [_passwordNameField setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_passwordNameField setDelegate:self];
    [_passwordNameField setSecureTextEntry:YES];
    [self.view addSubview:_passwordNameField];
  
    
    _signInButton = [UIButton buttonWithBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
    [_signInButton setTitleColor:theme.textColor forState:UIControlStateNormal];
    [_signInButton setTitle:@"Sign in" forState:UIControlStateNormal];
    [_signInButton setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_signInButton addTarget:self action:@selector(submitButtonPressed) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:_signInButton];
    
    _registerButton = [UIButton buttonWithBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
    [_registerButton setTitleColor:theme.textColor forState:UIControlStateNormal];
    [_registerButton setTitle:@"Sign up" forState:UIControlStateNormal];
    [_registerButton setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_registerButton addTarget:self action:@selector(registerButtonPressed) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:_registerButton];
    
    [self.view addConstraints:[self getConstraints]];
}

-(void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    [self.view setBackgroundColor:theme.backgroundColor];
    
    [_userNameField setBackgroundColor:theme.backgroundColor];
    [_userNameField setTextColor:theme.textColor];
    [_userNameField setFont:[UIFont fontWithName:theme.fontName size:20]];
    
    [_passwordNameField setBackgroundColor:theme.backgroundColor];
    [_passwordNameField setTextColor:theme.textColor];
    [_passwordNameField setFont:[UIFont fontWithName:theme.fontName size:20]];
    
    [_signInButton setButtonBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
    
    [_registerButton setButtonBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
}

-(void)viewDidAppear:(BOOL)animated{
    [super viewDidAppear:animated];

    [_userNameField becomeFirstResponder];
}

-(BOOL)textFieldShouldReturn:(UITextField *)textField{
    [textField resignFirstResponder];
    return YES;
}

-(void)submitButtonPressed{
    [_passwordNameField resignFirstResponder];
    [_userNameField resignFirstResponder];
    
    NSString *errorMessage = nil;
    if([_userNameField.text length] <= 0){
        errorMessage = @"Enter your username";
    }else if([_passwordNameField.text length] <= 0){
        errorMessage = @"Enter your password";
    }
    if(!errorMessage){
        [[UIApplication sharedApplication]setNetworkActivityIndicatorVisible:YES];
        [[User getInstance]attemptLoginWithUsername:_userNameField.text andPassword:_passwordNameField.text];
    }else{
        [[[UIAlertView alloc]initWithTitle:@"Uh oh!"
                                   message:errorMessage
                                  delegate:self
                         cancelButtonTitle:@"Ok"
                         otherButtonTitles:nil] show];
    }
    
}

-(void)registerButtonPressed{
    [_userNameField resignFirstResponder];
    [_passwordNameField resignFirstResponder];
    SignUpViewController *signUp = [[SignUpViewController alloc] init];
    [self.navigationController pushViewController:signUp animated:YES];
}

-(void)loginSuccessful{
    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       [[UIApplication sharedApplication]setNetworkActivityIndicatorVisible:NO];
                       [self.navigationController popViewControllerAnimated:YES];
                   });
}

-(void)authenticationFailed{
    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       [[UIApplication sharedApplication]setNetworkActivityIndicatorVisible:NO];
                       [[[UIAlertView alloc]initWithTitle:@"Uh oh!"
                                                  message:@"Bad username/password combo"
                                                 delegate:self
                                        cancelButtonTitle:@"Ok"
                                        otherButtonTitles:nil] show];
                   });
}



-(void)connectionFailed{
    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       [[UIApplication sharedApplication]setNetworkActivityIndicatorVisible:NO];
                       [[[UIAlertView alloc]initWithTitle:@"Uh oh!"
                                                  message:@"Could not connect"
                                                 delegate:self
                                        cancelButtonTitle:@"Ok"
                                        otherButtonTitles:nil] show];
                   });
}

-(NSArray *)getConstraints{
    NSMutableArray *constraints = [NSMutableArray arrayWithCapacity:5];
    
    NSLayoutConstraint *userNameCenter = [NSLayoutConstraint constraintWithItem:_userNameField
                                                                      attribute:NSLayoutAttributeCenterX
                                                                      relatedBy:NSLayoutRelationEqual
                                                                         toItem:self.view
                                                                      attribute:NSLayoutAttributeCenterX
                                                                     multiplier:1.0
                                                                       constant:0];
    [constraints addObject:userNameCenter];
    
    NSLayoutConstraint *userNameWidth = [NSLayoutConstraint constraintWithItem:_userNameField
                                                                     attribute:NSLayoutAttributeWidth
                                                                     relatedBy:NSLayoutRelationEqual
                                                                        toItem:self.view
                                                                     attribute:NSLayoutAttributeWidth
                                                                    multiplier:(2./3.)
                                                                      constant:0];
    [constraints addObject:userNameWidth];
    
    NSLayoutConstraint *userNameBottom = [NSLayoutConstraint constraintWithItem:_userNameField
                                                                      attribute:NSLayoutAttributeTop
                                                                      relatedBy:NSLayoutRelationEqual
                                                                         toItem:self.view
                                                                      attribute:NSLayoutAttributeCenterY
                                                                     multiplier:(1./3.)
                                                                       constant:0];
    [constraints addObject:userNameBottom];
    
    NSLayoutConstraint *userNameHeight = [NSLayoutConstraint constraintWithItem:_userNameField
                                                                      attribute:NSLayoutAttributeHeight
                                                                      relatedBy:NSLayoutRelationEqual
                                                                         toItem:self.view
                                                                      attribute:NSLayoutAttributeHeight
                                                                     multiplier:(1./12.)
                                                                       constant:0];
    [constraints addObject:userNameHeight];
    
    
    NSLayoutConstraint *passwordLeading = [NSLayoutConstraint constraintWithItem:_passwordNameField
                                                                       attribute:NSLayoutAttributeLeading
                                                                       relatedBy:NSLayoutRelationEqual
                                                                          toItem:_userNameField
                                                                       attribute:NSLayoutAttributeLeading
                                                                      multiplier:1.0
                                                                        constant:0];
    [constraints addObject:passwordLeading];
    
    NSLayoutConstraint *passwordTrailing = [NSLayoutConstraint constraintWithItem:_passwordNameField
                                                                        attribute:NSLayoutAttributeTrailing
                                                                        relatedBy:NSLayoutRelationEqual
                                                                           toItem:_userNameField
                                                                        attribute:NSLayoutAttributeTrailing
                                                                       multiplier:1.0
                                                                         constant:0];
    [constraints addObject:passwordTrailing];
    
    NSLayoutConstraint *passwordTop = [NSLayoutConstraint constraintWithItem:_passwordNameField
                                                                   attribute:NSLayoutAttributeTop
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:_userNameField
                                                                   attribute:NSLayoutAttributeBottom
                                                                  multiplier:1.0
                                                                    constant:PADDING/2];
    [constraints addObject:passwordTop];
    
    NSLayoutConstraint *passwordHeight = [NSLayoutConstraint constraintWithItem:_passwordNameField
                                                                      attribute:NSLayoutAttributeHeight
                                                                      relatedBy:NSLayoutRelationEqual
                                                                         toItem:_userNameField
                                                                      attribute:NSLayoutAttributeHeight
                                                                     multiplier:1.0
                                                                       constant:0];
    [constraints addObject:passwordHeight];
    
    NSLayoutConstraint *signInLeading = [NSLayoutConstraint constraintWithItem:_signInButton
                                                                     attribute:NSLayoutAttributeLeading
                                                                     relatedBy:NSLayoutRelationEqual
                                                                        toItem:_passwordNameField
                                                                     attribute:NSLayoutAttributeCenterX
                                                                    multiplier:1.0
                                                                      constant:PADDING/2];
    [constraints addObject:signInLeading];
    
    NSLayoutConstraint *signInTrailing = [NSLayoutConstraint constraintWithItem:_signInButton
                                                                    attribute:NSLayoutAttributeTrailing
                                                                    relatedBy:NSLayoutRelationEqual
                                                                       toItem:_passwordNameField
                                                                    attribute:NSLayoutAttributeTrailing
                                                                   multiplier:1.0
                                                                     constant:-PADDING/4];
    [constraints addObject:signInTrailing];
    
    NSLayoutConstraint *signInTop = [NSLayoutConstraint constraintWithItem:_signInButton
                                                                 attribute:NSLayoutAttributeTop
                                                                 relatedBy:NSLayoutRelationEqual
                                                                    toItem:_passwordNameField
                                                                 attribute:NSLayoutAttributeBottom
                                                                multiplier:1.0
                                                                  constant:PADDING];
    [constraints addObject:signInTop];
    
    NSLayoutConstraint *registerLeading = [NSLayoutConstraint constraintWithItem:_registerButton
                                                                       attribute:NSLayoutAttributeLeading
                                                                       relatedBy:NSLayoutRelationEqual
                                                                          toItem:_passwordNameField
                                                                       attribute:NSLayoutAttributeLeading
                                                                      multiplier:1.0
                                                                        constant:PADDING/4];
    [constraints addObject:registerLeading];
    
    NSLayoutConstraint *registerTrailing = [NSLayoutConstraint constraintWithItem:_registerButton
                                                                        attribute:NSLayoutAttributeTrailing
                                                                        relatedBy:NSLayoutRelationEqual
                                                                           toItem:_passwordNameField
                                                                        attribute:NSLayoutAttributeCenterX
                                                                       multiplier:1.0
                                                                         constant:-PADDING/2];
    [constraints addObject:registerTrailing];
    
    NSLayoutConstraint *registerAlign = [NSLayoutConstraint constraintWithItem:_registerButton
                                                                     attribute:NSLayoutAttributeTop
                                                                     relatedBy:NSLayoutRelationEqual
                                                                        toItem:_signInButton
                                                                     attribute:NSLayoutAttributeTop
                                                                    multiplier:1.0
                                                                      constant:0];
    [constraints addObject:registerAlign];
    
    return constraints;
}

-(void)dealloc{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
}

@end
