//
//  SignUpViewController.m
//  LittleNote
//
//  Created by Clark Barry on 1/25/13.
//  Copyright (c) 2013 CSHaus. All rights reserved.
//

#import "SignUpViewController.h"
#import "User.h"
#import "CommonDefines.h"
#import "UIButton+CustomColoring.h"
#import "ThemeManager.h"
#import "LittleNoteTheme.h"

@interface SignUpViewController ()
-(void)submitButtonPressed;
-(BOOL)validateEmail:(NSString *)candidate;
-(void)createSuccessful;
-(void)createFailed:(NSNotification *)notification;
-(void)connectionFailed;
-(void)showErrorWithMessage:(NSString *)message;
-(NSArray *)getConstraints;

@end

@implementation SignUpViewController

-(id)init{
    self = [super init];
    if(self){
        self.title = @"Sign up";
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(createSuccessful) name:kUserCreateSuccessfulNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(createFailed:) name:kUserCreateFailedNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectionFailed) name:kConnectionFailedNotification object:nil];
    }
    return self;
}

-(void)submitButtonPressed{
    User *user = [User getInstance];
    NSString *username = _userNameField.text;
    NSString *email = _emailField.text;
    NSString *password = _passwordField.text;
    NSString *confirmation = _confirmationField.text;
    
    NSString *errorMessage;
    if(![self validateEmail:email]){
        errorMessage = @"Invalid email";
    }else if(![password isEqualToString:confirmation]){
        errorMessage = @"Passwords don't match";
    }
    
    if(!errorMessage){
        [user attemptCreateUserWith:username Password:password andEmail:email];
    }else{
        [self showErrorWithMessage:errorMessage];
    }
}

-(BOOL)validateEmail:(NSString *)candidate{
    NSString *emailRegex = @"[A-Z0-9a-z._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,6}";
    NSPredicate *emailTest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", emailRegex];
    return [emailTest evaluateWithObject:candidate];
}

-(void)createSuccessful{
    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       [self.navigationController popToRootViewControllerAnimated:YES];
                   });
}

-(void)createFailed:(NSNotification *)notification{

}

-(void)connectionFailed{

}

-(void)showErrorWithMessage:(NSString *)message{
    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       [[[UIAlertView alloc]initWithTitle:@"Uh oh!"
                                                  message:message
                                                 delegate:self
                                        cancelButtonTitle:@"Ok"
                                        otherButtonTitles:nil] show];
                   });

}

-(void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    [self.view setBackgroundColor:theme.backgroundColor];
    [_userNameField setBackgroundColor:theme.backgroundColor];
    [_userNameField setTextColor:theme.textColor];
    
    [_emailField setBackgroundColor:theme.backgroundColor];
    [_emailField setTextColor:theme.textColor];
    
    [_passwordField setBackgroundColor:theme.backgroundColor];
    [_passwordField setTextColor:theme.textColor];
    
    [_confirmationField setBackgroundColor:theme.backgroundColor];
    [_confirmationField setTextColor:theme.textColor];
    
    [_submitButton setButtonBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
    
}
-(void)loadView{
    [super loadView];
    
    self.tabBarController.navigationItem.rightBarButtonItem = nil;
    
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    
    _userNameField = [[UITextField alloc]init];
    [_userNameField setPlaceholder:@"username"];
    [_userNameField setAutocapitalizationType:UITextAutocapitalizationTypeNone];
    [_userNameField setAutocorrectionType:UITextAutocorrectionTypeNo];
    [_userNameField setSpellCheckingType:UITextSpellCheckingTypeNo];
    [_userNameField setBorderStyle:UITextBorderStyleRoundedRect];
    [_userNameField setFont:[UIFont fontWithName:theme.fontName size:20]];
    [_userNameField setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_userNameField setDelegate:self];
    [self.view addSubview:_userNameField];
    
    _emailField = [[UITextField alloc]init];
    [_emailField setKeyboardType:UIKeyboardTypeEmailAddress];
    [_emailField setPlaceholder:@"email"];
    [_emailField setAutocapitalizationType:UITextAutocapitalizationTypeNone];
    [_emailField setAutocorrectionType:UITextAutocorrectionTypeNo];
    [_emailField setSpellCheckingType:UITextSpellCheckingTypeNo];
    [_emailField setBorderStyle:UITextBorderStyleRoundedRect];
    [_emailField setFont:[UIFont fontWithName:theme.fontName size:20]];
    [_emailField setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_emailField setDelegate:self];
    [self.view addSubview:_emailField];
    

    _passwordField = [[UITextField alloc]init];
    [_passwordField setBorderStyle:UITextBorderStyleRoundedRect];
    [_passwordField setFont:[UIFont fontWithName:theme.fontName size:20]];
    [_passwordField setPlaceholder:@"password"];
    [_passwordField setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_passwordField setDelegate:self];
    [_passwordField setSecureTextEntry:YES];
    [self.view addSubview:_passwordField];
    
    _confirmationField = [[UITextField alloc]init];
    [_confirmationField setBorderStyle:UITextBorderStyleRoundedRect];
    [_confirmationField setFont:[UIFont fontWithName:theme.fontName size:20]];
    [_confirmationField setPlaceholder:@"confirm"];
    [_confirmationField setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_confirmationField setDelegate:self];
    [_confirmationField setSecureTextEntry:YES];
    [self.view addSubview:_confirmationField];
    
    _submitButton = [UIButton buttonWithBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
    [_submitButton setTitleColor:theme.textColor forState:UIControlStateNormal];
    [_submitButton setTitle:@"Sign up!" forState:UIControlStateNormal];
    [_submitButton setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_submitButton addTarget:self action:@selector(submitButtonPressed) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:_submitButton];
    
    [self.view addConstraints:[self getConstraints]];
}

-(BOOL)textFieldShouldReturn:(UITextField *)textField{
    [textField resignFirstResponder];
    return YES;
}

-(NSArray *)getConstraints{
    NSMutableArray *constraints = [NSMutableArray array];
    NSLayoutConstraint *usernameCenter = [NSLayoutConstraint constraintWithItem:_userNameField
                                                                      attribute:NSLayoutAttributeCenterX
                                                                      relatedBy:NSLayoutRelationEqual
                                                                         toItem:self.view
                                                                      attribute:NSLayoutAttributeCenterX
                                                                     multiplier:1.0
                                                                       constant:0];
    [constraints addObject:usernameCenter];
    
    NSLayoutConstraint *usernameWidth = [NSLayoutConstraint constraintWithItem:_userNameField
                                                                     attribute:NSLayoutAttributeWidth
                                                                     relatedBy:NSLayoutRelationEqual
                                                                        toItem:self.view
                                                                     attribute:NSLayoutAttributeWidth
                                                                    multiplier:(2./3.)
                                                                      constant:0];
    [constraints addObject:usernameWidth];
    
    NSLayoutConstraint *usernameTop = [NSLayoutConstraint constraintWithItem:_userNameField
                                                                   attribute:NSLayoutAttributeTop
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:self.view
                                                                   attribute:NSLayoutAttributeTop
                                                                  multiplier:1.0
                                                                    constant:PADDING];
    [constraints addObject:usernameTop];
    
    NSLayoutConstraint *usernameHeight = [NSLayoutConstraint constraintWithItem:_userNameField
                                                                      attribute:NSLayoutAttributeHeight
                                                                      relatedBy:NSLayoutRelationEqual
                                                                         toItem:self.view
                                                                      attribute:NSLayoutAttributeHeight
                                                                     multiplier:(1./12.)
                                                                       constant:0];
    [constraints addObject:usernameHeight];
    
    NSLayoutConstraint *emailHeight = [NSLayoutConstraint constraintWithItem:_emailField
                                                                   attribute:NSLayoutAttributeHeight
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:_userNameField
                                                                   attribute:NSLayoutAttributeHeight
                                                                  multiplier:1.0
                                                                    constant:0];
    [constraints addObject:emailHeight];
    
    NSLayoutConstraint *emailWidth = [NSLayoutConstraint constraintWithItem:_emailField
                                                                  attribute:NSLayoutAttributeWidth
                                                                  relatedBy:NSLayoutRelationEqual
                                                                     toItem:_userNameField
                                                                  attribute:NSLayoutAttributeWidth
                                                                 multiplier:1.0
                                                                   constant:0];
    [constraints addObject:emailWidth];
    
    NSLayoutConstraint *centerEmail = [NSLayoutConstraint constraintWithItem:_emailField
                                                                   attribute:NSLayoutAttributeCenterX
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:self.view
                                                                   attribute:NSLayoutAttributeCenterX
                                                                  multiplier:1.0
                                                                    constant:0];
    [constraints addObject:centerEmail];
    
    NSLayoutConstraint *emailTop = [NSLayoutConstraint constraintWithItem:_emailField
                                                                attribute:NSLayoutAttributeTop
                                                                relatedBy:NSLayoutRelationEqual
                                                                   toItem:_userNameField
                                                                attribute:NSLayoutAttributeBottom
                                                               multiplier:1.0
                                                                 constant:PADDING/2];
    [constraints addObject:emailTop];
    
    NSLayoutConstraint *passwordHeight = [NSLayoutConstraint constraintWithItem:_passwordField
                                                                   attribute:NSLayoutAttributeHeight
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:_userNameField
                                                                   attribute:NSLayoutAttributeHeight
                                                                  multiplier:1.0
                                                                    constant:0];
    [constraints addObject:passwordHeight];
    
    NSLayoutConstraint *passwordWidth = [NSLayoutConstraint constraintWithItem:_passwordField
                                                                  attribute:NSLayoutAttributeWidth
                                                                  relatedBy:NSLayoutRelationEqual
                                                                     toItem:_userNameField
                                                                  attribute:NSLayoutAttributeWidth
                                                                 multiplier:1.0
                                                                   constant:0];
    [constraints addObject:passwordWidth];
    
    NSLayoutConstraint *centerpassword = [NSLayoutConstraint constraintWithItem:_passwordField
                                                                   attribute:NSLayoutAttributeCenterX
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:self.view
                                                                   attribute:NSLayoutAttributeCenterX
                                                                  multiplier:1.0
                                                                    constant:0];
    [constraints addObject:centerpassword];
    
    NSLayoutConstraint *passwordTop = [NSLayoutConstraint constraintWithItem:_passwordField
                                                                attribute:NSLayoutAttributeTop
                                                                relatedBy:NSLayoutRelationEqual
                                                                   toItem:_emailField
                                                                attribute:NSLayoutAttributeBottom
                                                               multiplier:1.0
                                                                 constant:PADDING/2];
    [constraints addObject:passwordTop];
    
    NSLayoutConstraint *confirmationHeight = [NSLayoutConstraint constraintWithItem:_confirmationField
                                                                      attribute:NSLayoutAttributeHeight
                                                                      relatedBy:NSLayoutRelationEqual
                                                                         toItem:_userNameField
                                                                      attribute:NSLayoutAttributeHeight
                                                                     multiplier:1.0
                                                                       constant:0];
    [constraints addObject:confirmationHeight];
    
    NSLayoutConstraint *confirmationWidth = [NSLayoutConstraint constraintWithItem:_confirmationField
                                                                     attribute:NSLayoutAttributeWidth
                                                                     relatedBy:NSLayoutRelationEqual
                                                                        toItem:_userNameField
                                                                     attribute:NSLayoutAttributeWidth
                                                                    multiplier:1.0
                                                                      constant:0];
    [constraints addObject:confirmationWidth];
    
    NSLayoutConstraint *centerconfirmation = [NSLayoutConstraint constraintWithItem:_confirmationField
                                                                      attribute:NSLayoutAttributeCenterX
                                                                      relatedBy:NSLayoutRelationEqual
                                                                         toItem:self.view
                                                                      attribute:NSLayoutAttributeCenterX
                                                                     multiplier:1.0
                                                                       constant:0];
    [constraints addObject:centerconfirmation];
    
    NSLayoutConstraint *confirmationTop = [NSLayoutConstraint constraintWithItem:_confirmationField
                                                                   attribute:NSLayoutAttributeTop
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:_passwordField
                                                                   attribute:NSLayoutAttributeBottom
                                                                  multiplier:1.0
                                                                    constant:PADDING/2];
    [constraints addObject:confirmationTop];

    NSLayoutConstraint *buttonHeight = [NSLayoutConstraint constraintWithItem:_submitButton
                                                                          attribute:NSLayoutAttributeHeight
                                                                          relatedBy:NSLayoutRelationEqual
                                                                             toItem:_userNameField
                                                                          attribute:NSLayoutAttributeHeight
                                                                         multiplier:1.0
                                                                           constant:0];
    [constraints addObject:buttonHeight];
    
    NSLayoutConstraint *buttonWidth = [NSLayoutConstraint constraintWithItem:_submitButton
                                                                         attribute:NSLayoutAttributeWidth
                                                                         relatedBy:NSLayoutRelationEqual
                                                                            toItem:_userNameField
                                                                         attribute:NSLayoutAttributeWidth
                                                                        multiplier:1.0
                                                                          constant:0];
    [constraints addObject:buttonWidth];
    
    NSLayoutConstraint *centerbutton = [NSLayoutConstraint constraintWithItem:_submitButton
                                                                          attribute:NSLayoutAttributeCenterX
                                                                          relatedBy:NSLayoutRelationEqual
                                                                             toItem:self.view
                                                                          attribute:NSLayoutAttributeCenterX
                                                                         multiplier:1.0
                                                                           constant:0];
    [constraints addObject:centerbutton];
    
    NSLayoutConstraint *buttonTop = [NSLayoutConstraint constraintWithItem:_submitButton
                                                                       attribute:NSLayoutAttributeTop
                                                                       relatedBy:NSLayoutRelationEqual
                                                                          toItem:_confirmationField
                                                                       attribute:NSLayoutAttributeBottom
                                                                      multiplier:1.0
                                                                        constant:PADDING/2];
    [constraints addObject:buttonTop];
    
    
    
    return constraints;
}

-(void)dealloc{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
}


@end
