//
//  UserViewController.m
//  LittleNote
//
//  Created by Clark Barry on 1/31/13.
//  Copyright (c) 2013 CSHaus. All rights reserved.
//

#import "UserViewController.h"
#import "UIImage+ResizeImage.h"
#import "SettingsViewController.h"
#import "ThemeManager.h"
#import "LittleNoteTheme.h"
#import "User.h"

@interface UserViewController ()
-(void)showSettings;
-(void)userDidLogout;
@end

@implementation UserViewController

-(id)init{
    self = [super init];
    if(self){
         [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(userDidLogout) name:kUserLogoutNotification object:nil];
    }
    return self;
}

-(void)showSettings{
    SettingsViewController *settings = [[SettingsViewController alloc]init];
    settings.delegate = self;
    UINavigationController *settingsNavigationController = [[UINavigationController alloc] initWithRootViewController:settings];
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    [settingsNavigationController.navigationBar setTintColor:theme.navBarColor];
    [self.tabBarController presentViewController:settingsNavigationController animated:YES completion:nil];
}

-(void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    if(!self.tabBarController.navigationItem.rightBarButtonItem){
        UIBarButtonItem *rightItem = [[UIBarButtonItem alloc] initWithImage:[UIImage imageWithImage:[UIImage imageNamed:@"SettingsIcon.png"]
                                                                                       scaledToSize:CGSizeMake(25, 25)]
                                                                      style:UIBarButtonItemStyleBordered
                                                                     target:self
                                                                     action:@selector(showSettings)];
        [self.tabBarController.navigationItem setRightBarButtonItem:rightItem animated:animated];
    }
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    [self.tabBarController.navigationController.navigationBar setTintColor:theme.navBarColor];
    
}

-(void)settingsDidFinish:(SettingsViewController *)settings{
    [self.tabBarController dismissViewControllerAnimated:YES completion:nil];
}

-(void)userDidLogout{
    self.tabBarController.navigationItem.rightBarButtonItem = nil;
}

-(void)dealloc{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
}

@end
