//
//  WriteViewController.m
//  ForClark
//
//  Created by Clark Barry on 12/21/12.
//  Copyright (c) 2012 CSHaus. All rights reserved.
//

#import "WriteViewController.h"

#import "URLFeeder.h"
#import "UIView+UIViewConvenience.h"
#import "NoteManager.h"
#import "UIButton+CustomColoring.h"
#import "User.h"
#import "Note.h"
#import "UITextView+FadeTextView.h"
#import "LittleNoteTheme.h"
#import "ThemeManager.h"
#import "CommonDefines.h"

@interface WriteViewController (Internal)
-(NSArray *)getConstraints;
-(void)noteEntryFinished;
-(void)submitButtonPressed;
-(void)noteSubmitFinished;
-(void)noteSubmitFailed;
@end

@implementation WriteViewController

-(id)init{
    self = [super init];
    if(self){
        self.title = @"Write";
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(noteSubmitFinished) name:kNoteSubmitFinishedNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(noteSubmitFailed) name:kNoteSubmitFailedNotification object:nil];
        [self.tabBarItem setImage:[UIImage imageNamed:@"LittleNoteWriteIcon.png"]];
    }
    return self;
}

-(void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    LittleNoteTheme *theme = [[ThemeManager getInstance] currentTheme];
    
    [self.view setBackgroundColor:theme.backgroundColor];
    
    [_textView setTextColor:theme.textColor];
    [_textView setBackgroundColor:theme.backgroundColor];
    [_textView setFont:[UIFont fontWithName:theme.fontName size:20]];
    
    [_submitButton setButtonBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
    [_submitButton setTitleColor:theme.textColor forState:UIControlStateNormal];
    
}

-(void)loadView{
    [super loadView];
    
    LittleNoteTheme *theme = [[ThemeManager getInstance] currentTheme];
    
    _textView = [[UITextView alloc] init];
    [_textView setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_textView addCornerRadius:5];
    [_textView addBorderWithColor:[UIColor darkGrayColor] andRadius:1.0];
    [_textView setFont:[UIFont fontWithName:theme.fontName size:20]];
    [_textView setDelegate:self];
    [self.view addSubview:_textView];
    

    _submitButton = [UIButton buttonWithBackgroundColor:theme.backgroundColor andHighlightColor:theme.highlightColor];
    [_submitButton setTitle:@"Submit" forState:UIControlStateNormal];
    [_submitButton setTranslatesAutoresizingMaskIntoConstraints:NO];
    [_submitButton addTarget:self action:@selector(submitButtonPressed) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:_submitButton];
    
    [self.view addConstraints:[self getConstraints]];
}

-(void)noteSubmitFinished{
    [_textView fadeInText:@""];
}
-(void)noteSubmitFailed{
    dispatch_async(dispatch_get_main_queue(), ^{
        [[[UIAlertView alloc]initWithTitle:@"Error"
                                   message:@"Couldn't send note"
                                  delegate:self
                         cancelButtonTitle:@"OK"
                         otherButtonTitles: nil]show];
    });
}


- (BOOL)textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text{
    if ([text isEqualToString:@"\n"]){
        [textView resignFirstResponder];
        return NO;
    }
    return YES;
}

-(void)doneButtonPressed{
    [_textView resignFirstResponder];
}

-(void)submitButtonPressed{
    [[UIApplication sharedApplication]setNetworkActivityIndicatorVisible:YES];
    [_textView resignFirstResponder];
    Note *newNote = [[Note alloc] initWithText:_textView.text andIndex:0];
    [[NoteManager getInstance] submitNote:newNote];

}

-(NSArray *)getConstraints{
    NSMutableArray *constraints = [NSMutableArray arrayWithCapacity:5];
    
    NSLayoutConstraint *topTextField = [NSLayoutConstraint constraintWithItem:_textView
                                                                    attribute:NSLayoutAttributeTop
                                                                    relatedBy:NSLayoutRelationEqual
                                                                       toItem:self.view
                                                                    attribute:NSLayoutAttributeTop
                                                                   multiplier:1.0
                                                                     constant:PADDING];
    [constraints addObject:topTextField];
    
    NSLayoutConstraint *paddingTextField = [NSLayoutConstraint constraintWithItem:_textView
                                                                        attribute:NSLayoutAttributeLeading
                                                                        relatedBy:NSLayoutRelationEqual
                                                                           toItem:self.view
                                                                        attribute:NSLayoutAttributeLeading
                                                                       multiplier:1.0
                                                                         constant:PADDING];
    [constraints addObject:paddingTextField];
    
    NSLayoutConstraint *widthTextField = [NSLayoutConstraint constraintWithItem:_textView
                                                                      attribute:NSLayoutAttributeWidth
                                                                      relatedBy:NSLayoutRelationEqual
                                                                         toItem:self.view
                                                                      attribute:NSLayoutAttributeWidth
                                                                     multiplier:1.0
                                                                       constant:-(PADDING*2)];
    [constraints addObject:widthTextField];
    
    NSLayoutConstraint *heightTextField = [NSLayoutConstraint constraintWithItem:_textView
                                                                       attribute:NSLayoutAttributeHeight
                                                                       relatedBy:NSLayoutRelationEqual
                                                                          toItem:self.view
                                                                       attribute:NSLayoutAttributeHeight
                                                                      multiplier:(9./20.)
                                                                        constant:0];
    [constraints addObject:heightTextField];
    
    NSLayoutConstraint *topSubmitButton = [NSLayoutConstraint constraintWithItem:_submitButton
                                                                 attribute:NSLayoutAttributeTop
                                                                 relatedBy:NSLayoutRelationEqual
                                                                    toItem:_textView
                                                                 attribute:NSLayoutAttributeBottom
                                                                multiplier:1.0
                                                                  constant:PADDING];
    [constraints addObject:topSubmitButton];
    
    NSLayoutConstraint *heightSubmitButton = [NSLayoutConstraint constraintWithItem:_submitButton
                                                                          attribute:NSLayoutAttributeHeight
                                                                          relatedBy:NSLayoutRelationEqual
                                                                             toItem:self.view
                                                                          attribute:NSLayoutAttributeHeight
                                                                         multiplier:(1./16.)
                                                                           constant:0];
    [constraints addObject:heightSubmitButton];
    
    NSLayoutConstraint *widthSubmitButton = [NSLayoutConstraint constraintWithItem:_submitButton
                                                                         attribute:NSLayoutAttributeWidth
                                                                         relatedBy:NSLayoutRelationEqual
                                                                            toItem:_textView
                                                                         attribute:NSLayoutAttributeWidth
                                                                        multiplier:(2./3.)
                                                                          constant:0];
    [constraints addObject:widthSubmitButton];
    
    NSLayoutConstraint *centerSubmitButton = [NSLayoutConstraint constraintWithItem:_submitButton
                                                                          attribute:NSLayoutAttributeCenterX
                                                                          relatedBy:NSLayoutRelationEqual
                                                                             toItem:self.view
                                                                          attribute:NSLayoutAttributeCenterX
                                                                         multiplier:1.0
                                                                           constant:0];
    [constraints addObject:centerSubmitButton];
    
    
    return constraints;
}

-(void)dealloc{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
}

@end
