//
//  NoteView.m
//  ForClark
//
//  Created by Clark Barry on 12/22/12.
//  Copyright (c) 2012 CSHaus. All rights reserved.
//

#import "NoteView.h"
#import "Note.h"

#import "CommonDefines.h"
#import "UILabel+FadeLabel.h"
#import "UIView+UIViewConvenience.h"
#import "UIView+UIViewDebug.h"
#import "ThemeManager.h"
#import "LittleNoteTheme.h"

@interface NoteView (Internal)
-(void)setUpViews;
-(void)prepForShow;
@end

@implementation NoteView

-(void)setUpViews{
    _imageView = [[UIImageView alloc] initWithFrame:self.bounds];
    [_imageView setContentMode:UIViewContentModeScaleAspectFit];
    [_imageView setImage:[UIImage imageNamed:@"NoNoteImage.png"]];
    [self addSubview:_imageView];
    _textView = [[UILabel alloc] initWithFrame:self.bounds];
    [_textView setUserInteractionEnabled:NO];
    [_textView setLineBreakMode:NSLineBreakByWordWrapping];
    
    CGPoint newOrigin = _textView.frame.origin;
    newOrigin.x += 5;
    newOrigin.y += 5;
    [_textView setOrigin:newOrigin];
    
    [_textView setNumberOfLines:0];

    [self addSubview:_textView];
}


-(id)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    if(self){
        [self setUpViews];
    }
    return self;
}

-(id)init{
    self = [super init];
    if(self){
        [self setUpViews];
    }
    return self;
}

-(void)configureForNote:(Note *)note withSize:(CGSize)size{
    [self setSize:size];
    _textView.alpha = 0;
    _imageView.alpha = 0;
    if(note.text){
        
        [_textView setText:note.text];
        [self prepForShow];
        
        _textView.alpha = 1;
    }
    if(note.image){
        [_imageView setSize:self.frame.size];
        [_imageView setImage:note.image];
        _imageView.alpha = 1;
        [self bringSubviewToFront:_imageView];
    }else{
        [self bringSubviewToFront:_textView];
    }
}


-(void)prepForShow{
    LittleNoteTheme *theme = [ThemeManager getInstance].currentTheme;
    
    [_textView setBackgroundColor:theme.backgroundColor];
    [_textView setTextColor:theme.textColor];
    [_textView setFont:[UIFont fontWithName:theme.fontName size:30]];
    
    [_textView sizeToFit];
    [_textView setWidth:self.frame.size.width-10];

    [_imageView setSize:self.frame.size];
}

-(void)configureForBlank:(CGSize)size{
    _imageView.alpha = 0;
    [_textView setText:@""];
    [self prepForShow];
}

-(void)configureForEmptyWithSize:(CGSize)size{
    [self setSize:size];

    _textView.alpha = 1;
    _imageView.alpha = 1;
    [self bringSubviewToFront:_imageView];
    [_textView setText:@"No notes here!"];
    [_imageView setImage:[UIImage imageNamed:@"NoNoteImage.png"]];
    
    [self prepForShow];
}
-(void)configureForNotSharing:(CGSize)size{
    [self setSize:size];
    _textView.alpha = 1;
    _imageView.alpha = 1;
    [self bringSubviewToFront:_imageView];
    [_textView setText:@"Find someone to share with!"];
    [_imageView setImage:[UIImage imageNamed:@"NotSharing.png"]];
    [self prepForShow];
}
-(void)configureForRefresh:(CGSize)size{
    [self setSize:size];
    _textView.alpha = 1;
    _imageView.alpha = 0;
    [_textView setText:@"Check for new notes?"];
    [self prepForShow];
}

-(void)showChecking{
    [_textView fadeInText:@"Checking for new notes"];
}

-(void)finishedChecking:(BOOL)withNewNotes{
    NSString *newText = withNewNotes ? @"Found new notes!" : @"Nothing new, come back later!";
    [_textView fadeInText:newText];
}

-(void)fadeInNote:(Note *)note withSize:(CGSize)size{
    [self prepForShow];
    [_textView fadeInText:note.text];
    
}

@end
